//This program code is released under GNU General Public License (GPL)
//(http://www.gnu.org/copyleft/gpl.html)
//(C) Wolfgang Gossel (2010)
// Muehlweg 49 
// D-06114 Halle
//or: Martin Luther University
//    Dept. Hydrogeology and Environmental Geology
//    V.-Seckendorff Platz 3
//    D-06120 Halle
//email: wolfgang.gossel@geo.uni-halle.de
//homepage: http://www.mo2geo.org

#include <iostream>
#include <fstream>
#include <string>
#include <math.h>
#include "calc_normals.h"

using namespace std;

void calcNormals(long nFaces, long vID1[], long vID2[], long vID3[], double pX[], double pY[], double pZ[], double nX[], double nY[], double nZ[])
{
        long counterFaces=0, a=0, b=0, c=0; 
        double paX=0, paY=0, paZ=0;
        double pbX=0, pbY=0, pbZ=0;
        double pcX=0, pcY=0, pcZ=0;
        double n1X=0, n1Y=0, n1Z=0; //normals not normalized
        double nnX=0, nnY=0, nnZ=0; //normalized normals
        
        #pragma omp parallel
        {
            #pragma omp for private counterFaces
        for(counterFaces=0;counterFaces<nFaces-1;counterFaces++)
        {
            paX=pX[vID1[counterFaces]];
            paY=pY[vID1[counterFaces]];
            paZ=pZ[vID1[counterFaces]];
            pbX=pX[vID2[counterFaces]];
            pbY=pY[vID2[counterFaces]];
            pbZ=pZ[vID2[counterFaces]];
            pcX=pX[vID3[counterFaces]];
            pcY=pY[vID3[counterFaces]];
            pcZ=pZ[vID3[counterFaces]];
            
            n1X=((paY-pbY)*(paZ+pbZ))+((pbY-pcY)*(pbZ+pcZ))+((pcY-paY)*(pcZ+paZ));
            n1Y=((paZ-pbZ)*(paX+pbX))+((pbZ-pcZ)*(pbX+pcX))+((pcZ-paZ)*(pcX+paX));
            n1Z=((paX-pbX)*(paY+pbY))+((pbX-pcX)*(pbY+pcY))+((pcX-paX)*(pcY+paY));

            nX[counterFaces]=n1X/(sqrt((n1X*n1X)+(n1Y*n1Y)+(n1Z*n1Z)));
            nZ[counterFaces]=n1Y/(sqrt((n1X*n1X)+(n1Y*n1Y)+(n1Z*n1Z)));
            nY[counterFaces]=n1Z/(sqrt((n1X*n1X)+(n1Y*n1Y)+(n1Z*n1Z)));
        }
        }
}
