//This program code is released under GNU General Public License (GPL)
//(http://www.gnu.org/copyleft/gpl.html)
//(C) Wolfgang Gossel (2010)
// Muehlweg 49 
// D-06114 Halle
//or: Martin Luther University
//    Dept. Hydrogeology and Environmental Geology
//    V.-Seckendorff Platz 3
//    D-06120 Halle
//email: wolfgang.gossel@geo.uni-halle.de
//homepage: http://www.mo2geo.org

#include <cstdlib>
#include "get_grid.h"
#include "create_faces.h"
#include "calc_normals.h"
#include "get_projectFile.h"
#include "zpr.h"
#include <math.h>
#include <string>
#include <gl/Gl.h>
#include <gl/Glu.h>
#include <gl/glut.h>

using namespace std;

void showTriangle(void);
void display(void);
void handleResize(int w, int h);
void myInit(void);

double *bounds;
long num_faces;
long numLayers;
long points_per_layer;
long points_per_slice;
long numberLayers;
double *pointX;
double *pointY;
double *pointZ;
long *vertexID1;
long *vertexID2;
long *vertexID3;
double *normalX;
double *normalY;
double *normalZ;
float *red;
float *green;
float *blue;
float *alpha;
double prjParams[5];

void pick(GLint name)
{
   printf("Pick: %d\n",name);
   fflush(stdout);
}

int main(int argc, char *argv[])
{
//long num_faces;
//long number_of_points;
long cols, rows, counter1, a;
double x;

getProjectFile projectData;
projectData.setFile(argv[1]);
projectData.getLinesProjectFile();
projectData.analyzeProjectFileContent();
numberLayers=projectData.getNumberSlices();
numLayers=numberLayers;
char *sliceFile[numLayers+1];

red=new float[numLayers+1];
green=new float[numLayers+1];
blue=new float[numLayers+1];
alpha=new float[numLayers+1];
for(int j=0;j<numLayers+1;j++)
{
    sliceFile[j]=projectData.getPathFileSlice(j);
    red[j]=(float(projectData.getRed(j)))/256;
    green[j]=(float(projectData.getGreen(j)))/256;
    blue[j]=(float(projectData.getBlue(j)))/256;
    alpha[j]=1;
}
projectData.getProjectionParams(prjParams);
//should be read from project file (after comparison of values)
//sliceFile[0]="d:\\layers\\t1_25.grd";
//sliceFile[1]="d:\\layers\\b1_25.grd";
//sliceFile[2]="d:\\layers\\b2_25.grd";
//sliceFile[3]="d:\\layers\\b3_25.grd";
//sliceFile[4]="d:\\layers\\b4_25.grd";

getGRID sliceGRID;
sliceGRID.setFile(sliceFile[0]);
sliceGRID.getPointsFromFile();
sliceGRID.analyze_file_content();
points_per_slice=sliceGRID.getNumberPoints();


double *pointX_sl=new double[points_per_slice];
double *pointY_sl=new double[points_per_slice];
double *pointZ_sl=new double[points_per_slice];

double *pointX_slice=new double[points_per_slice*(numLayers+1)];
double *pointY_slice=new double[points_per_slice*(numLayers+1)];
double *pointZ_slice=new double[points_per_slice*(numLayers+1)];

double *bounds=new double[6];

sliceGRID.getBoundaries(bounds);
cols=sliceGRID.getNumberCols();
rows=sliceGRID.getNumberRows();

for (long counterSlices=0; counterSlices<numLayers;counterSlices++)
{
    
    sliceGRID.setFile(sliceFile[counterSlices]);
    sliceGRID.getPointsFromFile();
    sliceGRID.analyze_file_content();
    sliceGRID.getPoints(pointX_sl, pointY_sl, pointZ_sl);
    
    for(long i=0; i<points_per_slice;i++)
    {
        pointX_slice[i+counterSlices*points_per_slice]=pointX_sl[i];
        pointY_slice[i+counterSlices*points_per_slice]=pointY_sl[i];
        pointZ_slice[i+counterSlices*points_per_slice]=pointZ_sl[i];
    }
}
pointX=new double[points_per_slice*(numLayers+1)];
pointY=new double[points_per_slice*(numLayers+1)];
pointZ=new double[points_per_slice*(numLayers+1)];
//bring midpoint of model area to 0/0

for (long counterSlices=0; counterSlices<numLayers;counterSlices++)
{
    for(long i=0;i<points_per_slice;i++)
    {
        pointX[i+counterSlices*points_per_slice]=pointX_slice[i+counterSlices*points_per_slice]-bounds[0]-((bounds[1]-bounds[0])/2);
        pointY[i+counterSlices*points_per_slice]=bounds[2]+((bounds[3]-bounds[2])/2)-pointY_slice[i+counterSlices*points_per_slice];
        pointZ[i+counterSlices*points_per_slice]=pointZ_slice[i+counterSlices*points_per_slice];
    }
}
genFaces gridFaces;
gridFaces.calcNumFacesFromGRID(cols,rows);
num_faces=gridFaces.getNumberFaces();
num_faces=2*num_faces+4*cols+4*rows;

long *v_ID1=new long[num_faces];
long *v_ID2=new long[num_faces];
long *v_ID3=new long[num_faces];

gridFaces.vertexFacesID(v_ID1, v_ID2, v_ID3);

vertexID1=new long[num_faces*(numLayers)];
vertexID2=new long[num_faces*(numLayers)];
vertexID3=new long[num_faces*(numLayers)];

for (long counterLayers=0; counterLayers<numLayers;counterLayers++)
{
    for(long i=0;i<num_faces;i++)
    {
        vertexID1[i+counterLayers*num_faces]=v_ID1[i]+(counterLayers*points_per_slice);
        vertexID2[i+counterLayers*num_faces]=v_ID2[i]+(counterLayers*points_per_slice);
        vertexID3[i+counterLayers*num_faces]=v_ID3[i]+(counterLayers*points_per_slice);
    }
}

normalX=new double[num_faces*numLayers];
normalY=new double[num_faces*numLayers];
normalZ=new double[num_faces*numLayers];

calcNormals(num_faces*numLayers, vertexID1, vertexID2, vertexID3, pointX, pointY, pointZ, normalX, normalY, normalZ);

glutInit(&argc, argv);
glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB);
glutInitWindowSize(1000, 800);
glutInitWindowPosition(100,150);
glutCreateWindow("Geol. Model");
glutDisplayFunc(display);
zprInit();
zprSelectionFunc(showTriangle);
//myInit();
zprPickFunc(pick);
glViewport(0,0,1000,800);
glutReshapeFunc(handleResize);

glutMainLoop();

return(0);
}


void showTriangle(void)
{   
    float pi=3.1415296;
    double scaleLength=0, scaleHeight=0;
    float *colorLayer1=new float[12];

    float refl[4]={0.5,0.3,0.8,0.4};
    long calcOffset;
    glPushMatrix();
//    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
      
    glTranslatef(0.0,0.0,-4.0);
    glRotatef(prjParams[1], 1.0, 0.0, 0.0 ); 
    glRotatef(prjParams[0], 0.0, 1.0, 0.0 ); 
    
    scaleLength=prjParams[2]; 
    scaleHeight=prjParams[3];
    
    glBegin(GL_TRIANGLES);
    glEnable(GL_COLOR_MATERIAL);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, refl);
    
    glPushMatrix();


    for (long l=0; l<numberLayers; l++)
    {
//        glColor4f(colorLayer1[0+(l*(numberLayers+1))], colorLayer1[1+(l*(numberLayers+1))], colorLayer1[2+(l*(numberLayers+1))], colorLayer1[3+(l*(numberLayers+1))]);
        glColor4f(red[l], green[l], blue[l], 1);
            
        for(long j=0;j<num_faces;j++)
        {   
            glPushMatrix();     
            long i=j+l*num_faces;
            calcOffset=numberLayers-l;
            glNormal3f(normalX[i],normalY[i],normalZ[i]);
            glVertex3f((pointX[vertexID1[i]])/scaleLength,(pointZ[vertexID1[i]]+prjParams[4]*calcOffset)/scaleHeight, (pointY[vertexID1[i]])/scaleLength);
            glVertex3f((pointX[vertexID2[i]])/scaleLength,(pointZ[vertexID2[i]]+prjParams[4]*calcOffset)/scaleHeight, (pointY[vertexID2[i]])/scaleLength);
            glVertex3f((pointX[vertexID3[i]])/scaleLength,(pointZ[vertexID3[i]]+prjParams[4]*calcOffset)/scaleHeight, (pointY[vertexID3[i]])/scaleLength);
            glPopMatrix();
        }
    }
    glPopMatrix();
    glEnd();
    
    glPopMatrix();

//    glutSwapBuffers();
}

void display(void)
{
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    showTriangle();
    glutSwapBuffers();
}

void handleResize(int width, int height) 
{
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(30.0f,width/height,0.1f,100.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	glClearDepth(1.0);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	glFrontFace(GL_CW);	

	glShadeModel(GL_SMOOTH);
	GLfloat AmbientLight[] = {0.3, 0.3, 0.3, 1.0};
	GLfloat DiffuseLight[] = {0.5, 0.5, 0.5, 1.0};
	GLfloat LightPosition[] = {0.0, 0.0, 0.0, 1.0};
	GLfloat SpecularLight[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat SpecularReflection[] = {0.4, 0.4, 0.4, 1.0};
	
	glEnable(GL_LIGHTING);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, AmbientLight);
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	
	glEnable(GL_LIGHTING);
	glLightfv(GL_LIGHT0,GL_AMBIENT, AmbientLight);
	glLightfv(GL_LIGHT0,GL_DIFFUSE, DiffuseLight);
	glLightfv(GL_LIGHT0,GL_SPECULAR, SpecularLight);
	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0,GL_POSITION,LightPosition);
	
//	glEnable(GL_COLOR_MATERIAL);
//	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
//	glMaterialfv(GL_FRONT, GL_SPECULAR, SpecularReflection);
//	glMateriali(GL_FRONT,GL_SHININESS,10);
}


void myInit(void)
{
    glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);				// Black Background
	glClearDepth(1.0f);									// Depth Buffer Setup
}

