#include <cstdlib>
#include "get_grid.h"
#include "create_faces.h"
#include "calc_normals.h"
#include <gl/Gl.h>
#include <gl/Glu.h>
#include <gl/glut.h>

using namespace std;

void showTriangle(void);
void gridShowInit(void);
void initRendering(void);
void handleResize(int w, int h);
void update(int value);


void setWindow(double left, double right, double bottom, double top)
{
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(left, right, bottom, top);
}

long num_faces;
long number_of_points;
double *pointX;
double *pointY;
double *pointZ;
long *vertexID1;
long *vertexID2;
long *vertexID3;
double *normalX;
double *normalY;
double *normalZ;

int main(int argc, char *argv[])
{
//long num_faces;
//long number_of_points;
long cols, rows, counter1, a;
double x;

getGRID inputGRID;
inputGRID.setFile();
inputGRID.getPointsFromFile();
inputGRID.analyze_file_content();
number_of_points=inputGRID.getNumberPoints();

//double *pointX=new double[number_of_points];
//double *pointY=new double[number_of_points];
//double *pointZ=new double[number_of_points];
pointX=new double[number_of_points];
pointY=new double[number_of_points];
pointZ=new double[number_of_points];

long *pointID=new long[number_of_points];

inputGRID.getPoints(pointX, pointY, pointZ);

cols=inputGRID.getNumberCols();
rows=inputGRID.getNumberRows();

genFaces gridFaces;
gridFaces.calcNumFacesFromGRID(cols,rows);
num_faces=gridFaces.getNumberFaces();

//long *vertexID1=new long[num_faces];
//long *vertexID2=new long[num_faces];
//long *vertexID3=new long[num_faces];

vertexID1=new long[num_faces];
vertexID2=new long[num_faces];
vertexID3=new long[num_faces];

gridFaces.vertexFacesID(vertexID1, vertexID2, vertexID3);

//double *normalX=new double[num_faces];
//double *normalY=new double[num_faces];
//double *normalZ=new double[num_faces];

normalX=new double[num_faces];
normalY=new double[num_faces];
normalZ=new double[num_faces];

calcNormals(num_faces, vertexID1, vertexID2, vertexID3, pointX, pointY, pointZ, normalX, normalY, normalZ);

glutInit(&argc, argv);
glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB);
glutInitWindowSize(1000, 800);
glutInitWindowPosition(0,0);
glutCreateWindow("Geol. Model");
gridShowInit();
initRendering();
glutDisplayFunc(showTriangle);
glutReshapeFunc(handleResize);
glutTimerFunc(25, update, 0);
glutMainLoop();

return(0);
}


void showTriangle(void)
{
    
    glClear(GL_COLOR_BUFFER_BIT);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef(0.0f, 0.0f, -10);
    glRotatef(30.0f, 1.0f, 0.0f, 0.0f); //Vertikalwinkel
	glRotatef(180, 0.0f, 1.0f, 0.0f); //-_angle Horizontal: 180=von Sden
	GLfloat ambientColor[] = {0.4f, 0.4f, 0.4f, 1.0f};
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientColor);
	
	GLfloat lightColor0[] = {0.6f, 0.6f, 0.6f, 1.0f};
	GLfloat lightPos0[] = {-0.5f, 0.8f, 0.1f, 0.0f};
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightColor0);
	glLightfv(GL_LIGHT0, GL_POSITION, lightPos0);
	
	glScalef(5.0f, 5.0f, 5.0f);
	glTranslatef(-(float)(1200 - 1) / 2, 0.0f, -(float)(1600 - 1) / 2);
	
	glColor3f(0.3f, 0.9f, 0.0f);
    glBegin(GL_TRIANGLES);
    for(int i=0;i<num_faces;i++)
    {
        glVertex3f((pointX[vertexID1[i]]-3569700)/10,pointZ[vertexID1[i]]/20, (pointY[vertexID1[i]]-5666700)/10);
        glVertex3f((pointX[vertexID2[i]]-3569700)/10,pointZ[vertexID2[i]]/20, (pointY[vertexID2[i]]-5666700)/10);
        glVertex3f((pointX[vertexID3[i]]-3569700)/10,pointZ[vertexID3[i]]/20, (pointY[vertexID3[i]]-5666700)/10);
        glNormal3f(normalX[i],normalY[i],normalZ[i]);
        //cout<<(pointX[vertexID1[i]]-3569700)/10<<", "<<pointZ[vertexID1[i]]/20<<", "<<(pointY[vertexID1[i]]-5666700)/10<<endl;
    }
    glEnd();
    //glFlush();
    glutSwapBuffers();
    cout<<"fertig"<<endl;
}

void gridShowInit(void)
{
    glClearColor(0.0,0.0,0.0,1.0);
    glColor3f(0.3f,0.4f,0.3f);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
}

void initRendering() 
{
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_NORMALIZE);
	glShadeModel(GL_SMOOTH);
}

void handleResize(int w, int h) 
{
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(4500.0, (double)w / (double)h, 1.0, 200.0);//45=Entfernung
}
void update(int value) 
{
	float angle=0;
     angle+= 1.0f;
	if (angle > 360) {
		angle -= 360;
	}
	
	glutPostRedisplay();
	glutTimerFunc(25, update, 0);
}
