//This program code is released under GNU General Public License (GPL)
//(http://www.gnu.org/copyleft/gpl.html)
//(C) Wolfgang Gossel (2010)
// Muehlweg 49 
// D-06114 Halle
//or: Martin Luther University
//    Dept. Hydrogeology and Environmental Geology
//    V.-Seckendorff Platz 3
//    D-06120 Halle
//email: wolfgang.gossel@geo.uni-halle.de
//homepage: http://www.mo2geo.org

#include <cstdlib>
#include "get_grid.h"
#include "get_projectFile.h"
#include <math.h>
#include <string>
#include <gl/Gl.h>
#include <gl/Glu.h>
#include <gl/glut.h>


using namespace std;

void showCrossSection(void);
void handleResize(int w, int h);
void myInit(void);
void RenderString(float x, float y, void *font, char* string);

double *bounds;
long num_faces;
long numLayers;
long points_per_layer;
long points_per_slice;
long numberLayers;
double *pointX;
double *pointY;
double *pointZ;
long *vertexID1;
long *vertexID2;
long *vertexID3;
float *red;
float *green;
float *blue;
float *alpha;
double prjParams[3];
double *zValue;
double *distanceCS;
string *slName;
double *vertexXCoo;
double *vertexYCoo;
long numRPAll;

int main(int argc, char *argv[])
{
//long num_faces;
//long number_of_points;
long cols, rows, counter1, a;
double x;
long numberCSVertices;

getProjectFile projectData;
projectData.setFile(argv[1]);
projectData.getLinesProjectFile();
projectData.analyzeProjectFileContent();
projectData.getLineFileContent();
projectData.analyzeLineFileContent();
numberCSVertices=projectData.getNumberVertices();
numberLayers=projectData.getNumberSlices();
numLayers=numberLayers;

char *sliceFile[numLayers+1];

red=new float[numLayers+1];
green=new float[numLayers+1];
blue=new float[numLayers+1];
alpha=new float[numLayers+1];
slName=new string[numLayers+1];

for(int j=0;j<numLayers;j++)
{
    sliceFile[j]=projectData.getPathFileSlice(j);
    slName[j]=projectData.getNameSlice(j);
    red[j]=(float(projectData.getRed(j)))/256;
    green[j]=(float(projectData.getGreen(j)))/256;
    blue[j]=(float(projectData.getBlue(j)))/256;
    alpha[j]=1;
}

projectData.getProjectionParams(prjParams);

vertexXCoo=new double[numberCSVertices];
vertexYCoo=new double[numberCSVertices];

for(int j=0;j<numberCSVertices;j++)
{
    vertexXCoo[j]=projectData.getVertexX(j);
    vertexYCoo[j]=projectData.getVertexY(j);
}

getGRID sliceGRID;
sliceGRID.setFile(sliceFile[0]);
sliceGRID.getPointsFromFile();
sliceGRID.analyze_file_content();
points_per_slice=sliceGRID.getNumberPoints();


double *pointX_sl=new double[points_per_slice];
double *pointY_sl=new double[points_per_slice];
double *pointZ_sl=new double[points_per_slice];

double *pointX_slice=new double[points_per_slice*(numLayers+1)];
double *pointY_slice=new double[points_per_slice*(numLayers+1)];
double *pointZ_slice=new double[points_per_slice*(numLayers+1)];

double *bounds=new double[6];

sliceGRID.getBoundaries(bounds);
cols=sliceGRID.getNumberCols();
rows=sliceGRID.getNumberRows();

for (long counterSlices=0; counterSlices<numLayers;counterSlices++)
{
    
    sliceGRID.setFile(sliceFile[counterSlices]);
    sliceGRID.getPointsFromFile();
    sliceGRID.analyze_file_content();
    sliceGRID.getPoints(pointX_sl, pointY_sl, pointZ_sl);
    
    for(long i=0; i<points_per_slice;i++)
    {
        pointX_slice[i+counterSlices*points_per_slice]=pointX_sl[i];
        pointY_slice[i+counterSlices*points_per_slice]=pointY_sl[i];
        pointZ_slice[i+counterSlices*points_per_slice]=pointZ_sl[i];
    }
}

//calculate indeces of vertices

long *colNum=new long[numberCSVertices];
long *rowNum=new long[numberCSVertices];
long *index=new long[numberCSVertices];

int counterVertices=0;
double xDiff=0, colWidth=0, yDiff=0, rowWidth=0;

for (counterVertices=0; counterVertices<numberCSVertices; counterVertices++)
{
    xDiff=(vertexXCoo[counterVertices]-bounds[0]);
    colWidth=(bounds[1]-bounds[0])/cols;
    colNum[counterVertices]=(long)(xDiff/colWidth);
    yDiff=(vertexYCoo[counterVertices]-bounds[2]);
    rowWidth=(bounds[3]-bounds[2])/rows;
    rowNum[counterVertices]=(long)(yDiff/rowWidth);
    index[counterVertices]=rowNum[counterVertices]*cols+colNum[counterVertices];   
}

//calculate difference of columns and rows between vertices and 
//calculate number of raster points (RP) to be selected along cross section

long *colDiff=new long[numberCSVertices];
long *rowDiff=new long[numberCSVertices];
long *numRP=new long[numberCSVertices];
long cDiff=0, rDiff=0; //numRPAll=0, 

for (counterVertices=0; counterVertices<numberCSVertices-1; counterVertices++)
{
    colDiff[counterVertices]=colNum[counterVertices+1]-colNum[counterVertices];
    cDiff=abs(colDiff[counterVertices]);
    rowDiff[counterVertices]=rowNum[counterVertices+1]-rowNum[counterVertices];
    rDiff=abs(rowDiff[counterVertices]);
    if (cDiff>rDiff)
    {
        numRP[counterVertices]=cDiff;
    }
    else
    {
        numRP[counterVertices]=rDiff;
    }
    numRPAll=numRPAll+numRP[counterVertices];
}

//calculate distances of vertices, distance to starting point of cross section and indeces of RP

long *selRPIndex= new long[numRPAll];
double *selRPDistTemp= new double[numRPAll];
double *selRPDist= new double[numRPAll];
long counterRPTotal=0, tempRPIndex=0, indexDiff=0;
double tempDistR=0, tempDistC=0, tempDist=0, distDiff=0;
selRPIndex[0]=0;
selRPDist[0]=0;
double vertexDist=0;

for (counterVertices=0; counterVertices<numberCSVertices-1; counterVertices++)
{
    for(long counterRP=0; counterRP<numRP[counterVertices];counterRP++)
    {
            tempRPIndex=index[counterVertices];
            tempRPIndex=tempRPIndex+(long(colDiff[counterVertices]*counterRP/numRP[counterVertices]+0.5));
            tempRPIndex=tempRPIndex+(long((rowDiff[counterVertices]*counterRP/numRP[counterVertices])*cols)+0.5);
            selRPIndex[counterRPTotal]=tempRPIndex;
            tempDistC=vertexXCoo[counterVertices+1]-vertexXCoo[counterVertices];
            tempDistC=tempDistC*tempDistC;
            tempDistR=vertexYCoo[counterVertices+1]-vertexYCoo[counterVertices];
            tempDistR=tempDistR*tempDistR;
            tempDist=sqrt(tempDistC+tempDistR);
            tempDist=tempDist/numRP[counterVertices];
            selRPDist[counterRPTotal]=selRPDist[counterRPTotal-1]+tempDist;
//            if (counterRPTotal>1)
//            {
//            indexDiff=selRPIndex[counterRPTotal]-selRPIndex[counterRPTotal-1];
//            distDiff=selRPDist[counterRPTotal]-selRPDist[counterRPTotal-1];
//            cout<<selRPDist[counterRPTotal]<<", "<<distDiff<<", "<<selRPIndex[counterRPTotal]<<", "<<indexDiff<<endl;
//            }
            counterRPTotal++;   
    }
    vertexDist=vertexDist+sqrt((vertexXCoo[counterVertices]*vertexXCoo[counterVertices])+(vertexYCoo[counterVertices]*vertexYCoo[counterVertices]));
    
}

zValue=new double[numLayers*numRPAll];
distanceCS=new double[numLayers*numRPAll];
long tempIndex;

for (long i=0;i<numLayers;i++)
{
    for (long j=0;j<numRPAll;j++)
    {
        tempIndex=selRPIndex[j];
        zValue[i*numRPAll+j]=pointZ_slice[tempIndex+i*points_per_slice]/prjParams[1];
        distanceCS[i*numRPAll+j]=selRPDist[j]/prjParams[0];
    }
}

glutInit(&argc, argv);
glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB);
glutInitWindowSize(1000, 800);
glutInitWindowPosition(100,150);
glutCreateWindow("Virtual Cross Section");
glutDisplayFunc(showCrossSection);
myInit();
glViewport(0,0,400,800);
//glutReshapeFunc(handleResize);
glutMainLoop();

return(0);

}


void showCrossSection(void)
{   

    float refl[4]={0.5,0.3,0.8,0.4};
    double z=0.9, offsetLeft=0.0;
    long j=0, l=0;
    
    offsetLeft=0.8;
    
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

    glBegin(GL_QUADS);
    glEnable(GL_COLOR_MATERIAL);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, refl);

    for (l=0; l<numberLayers; l++)
    {
        for(j=0;j<numRPAll-1;j++)
        {
            glColor4f(red[l], green[l], blue[l], 1);
            glVertex2f(distanceCS[j]-offsetLeft, zValue[j+l*numRPAll]);
            glVertex2f(distanceCS[j+1]-offsetLeft, zValue[j+l*numRPAll]);
            glVertex2f(distanceCS[j+1]-offsetLeft, zValue[j+(l+1)*numRPAll]);
            glVertex2f(distanceCS[j]-offsetLeft, zValue[j+(l+1)*numRPAll]);
            //glColor3f(0.0, 1.0, 0.0);
        }
        j++;
    }
    
    glEnd();

//    char heightSlice[7];
//    int isConv;
//    z=0.9;
//    double thickness=0;
//    for (int l=0; l<numberLayers; l++)
//    {
//        sprintf(heightSlice, "%4.2f", zValue[l]);
//        RenderString(-0.9, z, GLUT_BITMAP_9_BY_15, heightSlice);
//        z=z+(zValue[l+1]-zValue[l])/prjParams[2];
//    }
//    
//    z=0.9+(0.5*((zValue[1]-zValue[0])/prjParams[2]));
//    
//
//    char sliceNameA[50];
//    
//    for (int l=0; l<numberLayers-1; l++)
//    {
//        thickness=zValue[l]-zValue[l+1];
//        char *sliceNme = new char[slName[l].size()+1];
//        strcpy(sliceNme, slName[l].c_str());
//        sliceNme[slName[l].size()+1]='\0';
//        sprintf(sliceNameA, "%4.2f", thickness); 
//        strcat(sliceNameA,", ");
//        strcat(sliceNameA, sliceNme);      
//        RenderString(0.1, z, GLUT_BITMAP_9_BY_15, sliceNameA);
//        z=z+0.5*(zValue[l+1]-zValue[l])/prjParams[2]+0.5*(zValue[l+2]-zValue[l+1])/prjParams[2];
//    }
    glutSwapBuffers();

}

//void handleResize(int width, int height) 
//{
//	glViewport(0, 0, width, height);
//	glMatrixMode(GL_PROJECTION);
//	glLoadIdentity();
//	gluPerspective(30.0f,width/height,0.1f,100.0f);
//	glMatrixMode(GL_MODELVIEW);
//	glLoadIdentity();
//	
//	glClearDepth(1.0);
//	glEnable(GL_DEPTH_TEST);
//	glDepthFunc(GL_LEQUAL);
//	
//	glEnable(GL_CULL_FACE);
//	glCullFace(GL_BACK);
//	glFrontFace(GL_CW);	
//
//	glShadeModel(GL_SMOOTH);
//	GLfloat AmbientLight[] = {0.3, 0.3, 0.3, 1.0};
//	GLfloat DiffuseLight[] = {0.5, 0.5, 0.5, 1.0};
//	GLfloat LightPosition[] = {0.0, 0.0, 0.0, 1.0};
//	GLfloat SpecularLight[] = {1.0, 1.0, 1.0, 1.0};
//	GLfloat SpecularReflection[] = {0.4, 0.4, 0.4, 1.0};
//	
//	glEnable(GL_LIGHTING);
//	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, AmbientLight);
//	glEnable(GL_COLOR_MATERIAL);
//	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
//	
//	glEnable(GL_LIGHTING);
//	glLightfv(GL_LIGHT0,GL_AMBIENT, AmbientLight);
//	glLightfv(GL_LIGHT0,GL_DIFFUSE, DiffuseLight);
//	glLightfv(GL_LIGHT0,GL_SPECULAR, SpecularLight);
//	glEnable(GL_LIGHT0);
//	glLightfv(GL_LIGHT0,GL_POSITION,LightPosition);
//	
//}
//
//
void myInit(void)
{
    glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);				// Black Background
	glClearDepth(1.0f);									// Depth Buffer Setup
}


void RenderString(float x, float y, void *font, char* stringLayer)
{  
  char *c;

  glColor3f(1.0, 1.0, 1.0); 
  glRasterPos2f(x, y);
  
  for(char* c=stringLayer; *c!='\0';c++)
  {
        glutBitmapCharacter(font, *c);
    }
}


